package xyz.lychee.lagfixer.utils;

import java.time.Duration;
import java.util.HashMap;
import java.util.concurrent.atomic.AtomicReference;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.bukkit.ChatColor;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.bukkit.plugin.java.JavaPlugin;
import xyz.lychee.lagfixer.hooks.PlaceholderAPIHook;
import xyz.lychee.lagfixer.libs.adventure.text.Component;
import xyz.lychee.lagfixer.libs.adventure.text.TextComponent;
import xyz.lychee.lagfixer.libs.adventure.text.event.ClickEvent;
import xyz.lychee.lagfixer.libs.adventure.text.event.HoverEvent;
import xyz.lychee.lagfixer.libs.annotations.Nullable;
import xyz.lychee.lagfixer.managers.ConfigManager;

public class MessageUtils extends JavaPlugin {
   private static final HashMap<String, Character> replaceable = new HashMap();

   private static void add(String from, char to) {
      replaceable.put('{' + from + '}', to);
   }

   public static String fixColors(@Nullable CommandSender sender, String message) {
      AtomicReference<String> text = new AtomicReference(message);
      replaceable.forEach((from, to) -> {
         text.updateAndGet((s) -> {
            return s.replace(from, Character.toString(to));
         });
      });
      PlaceholderAPIHook papi = PlaceholderAPIHook.getInstance();
      return papi != null && papi.isLoaded() && sender instanceof Player ? papi.applyPlaceholders((Player)sender, ChatColor.translateAlternateColorCodes('&', (String)text.get())) : ChatColor.translateAlternateColorCodes('&', (String)text.get());
   }

   public static boolean sendMessage(boolean prefix, CommandSender sender, String message) {
      sender.sendMessage(fixColors(sender, (prefix ? ConfigManager.getInstance().getLegacyPrefix() : "") + "&f" + message));
      return false;
   }

   public static TextComponent formatString(CommandSender sender, String message) {
      TextComponent text = Component.empty();
      Pattern pattern = Pattern.compile("(https?://\\S+)");
      Matcher matcher = pattern.matcher(message);

      int lastEnd;
      String remainingText;
      for(lastEnd = 0; matcher.find(); lastEnd = matcher.end()) {
         remainingText = message.substring(lastEnd, matcher.start());
         if (!remainingText.isEmpty()) {
            text = (TextComponent)text.append(Component.text(fixColors(sender, remainingText)));
         }

         String url = fixColors(sender, matcher.group());
         text = (TextComponent)text.append(((TextComponent)Component.text(url).clickEvent(ClickEvent.openUrl(ChatColor.stripColor(url)))).hoverEvent(HoverEvent.showText(ConfigManager.getInstance().getPrefix().append(Component.text("Click to open url!")))));
      }

      remainingText = fixColors(sender, message.substring(lastEnd));
      if (!remainingText.isEmpty()) {
         text = (TextComponent)text.append(Component.text(remainingText));
      }

      return text;
   }

   public static TextComponent createMessage(boolean bool, TextComponent message) {
      return bool ? (TextComponent)((TextComponent)Component.empty().append(ConfigManager.getInstance().getPrefix())).append(message) : message;
   }

   public static boolean isInteger(String string) {
      return Pattern.matches("-?[0-9]+", string.subSequence(0, string.length()));
   }

   public static String getDurationBreakdown(long duration, boolean showMS) {
      if (duration <= 0L) {
         return showMS ? "0ms" : "0s";
      } else {
         StringBuilder sb = new StringBuilder();
         Duration d = Duration.ofMillis(duration);
         long days = d.toDays();
         if (days > 0L) {
            sb.append(days).append("d ");
         }

         long hours = d.toHours() % 24L;
         if (hours > 0L) {
            sb.append(hours).append("h ");
         }

         long minutes = d.toMinutes() % 60L;
         if (minutes > 0L) {
            sb.append(minutes).append("m ");
         }

         if (days > 0L) {
            return sb.deleteCharAt(sb.length() - 1).toString();
         } else {
            long seconds = d.toMillis() / 1000L % 60L;
            long millis = d.toMillis() % 1000L;
            if (seconds > 0L) {
               if (showMS && millis > 0L) {
                  sb.append(seconds).append(",").append(millis).append("s");
               } else {
                  sb.append(seconds).append("s");
               }
            } else if (showMS && millis > 0L) {
               sb.append(millis).append("ms");
            }

            return sb.toString();
         }
      }
   }

   static {
      add("*", '•');
      add(">>", '»');
      add("<<", '«');
   }
}
